# -*- coding: UTF-8 -*-
require 'AlmanacDateTime'
require 'AlmanacMath'

#
#= 二十八宿（Twenty Eight Mansions）算出用クラスライブラリ
#
#Authors::   Yoshihiko Hara(GoroNeko)
#Date::      2010/08/14
#Version::   0.9.0
#Copyright:: Copyright (C) Yoshihiko Hara(GoroNeko), 2010. All rights reserved.
#License::   GNU GENERAL PUBLIC LICENSE Version 2
#

class AlmanacTwentyEightMansions

  # 本クラスライブラリの、名前とバージョン
  NAME = "AlmanacTwentyEightMansions"
  VERSION = "0.9.0"

  # 二十八宿のそれぞれの節気を示す番号
  # 0(角)～27(軫)
  TWENTY_EIGHT_MANSIONS_NUMBER \
    = Array.new([0,1,2,3,4,5,6,
                 7,8,9,10,11,12,13,
                 14,15,16,17,18,19,20,
                 21,22,23,24,25,26,27])

  # 二十八宿の漢字和名
  # "氐" を "ﾃｲ" にした理由は、Windows OS コンソールに対して
  # Curses ライブラリを利用して表示を行おうとした際、文字化けを
  # おこすため
  TWENTY_EIGHT_MANSIONS_NAME_J \
    = Array.new(["角", "亢", "ﾃｲ", "房", "心", "尾", "箕",
                 "斗", "牛", "女", "虚", "危", "室", "壁",
                 "奎", "婁", "胃", "昴", "畢", "觜", "参",
                 "井", "鬼", "柳", "星", "張", "翼", "軫"])

  # 二十八宿の英名
  # ウィキペディア英語版の記事を参考にした
  # (http://en.wikipedia.org/wiki/Twenty-eight_mansions)
  TWENTY_EIGHT_MANSIONS_E \
    = Array.new(["Horn", "Neck", "Root", "Room", "Heart", "Tail", "Winnowing Basket",
                 "Dipper", "Ox", "Girl", "Emptiness", "Rooftop", "Encampment", "Wall",
                 "Legs", "Bond", "Stomach", "Hairy Head", "Net", "Turtle Beak", "Three Stars",
                 "Well", "Ghost", "Willow", "Star", "Extended Net", "Wings", "Chariot"])

  # 指定西暦年月日(twenty_eight_mansions_date_time:AlmanacDateTime)の
  # 二十八宿番号(twenty_eight_mansions_number:Integer)を返す
  #
  # 【引数】
  #   1.指定西暦年月日(twenty_eight_mansions_date_time:AlmanacDateTime)
  # 【戻り値】
  #   1.二十八宿番号(twenty_eight_mansions_number:Integer)
  #
  def get_twenty_eight_mansions_number(twenty_eight_mansions_date_time)

    # 指定年月日の時差を調整し、日本における日時に調整しておく
    twenty_eight_mansions_date_time \
      = twenty_eight_mansions_date_time.change_offset_to_japan(twenty_eight_mansions_date_time)

    # 指定年月日の正午の日時情報を求める
    target_date_time \
      = AlmanacDateTime.new(twenty_eight_mansions_date_time.year, \
                            twenty_eight_mansions_date_time.month, \
                            twenty_eight_mansions_date_time.day, \
                            12, 0, 0, \
                            twenty_eight_mansions_date_time.offset)

    # 修正後の日時から得たユリウス日をもとに二十八宿番号を求める
    twenty_eight_mansions_jd = target_date_time.ajd
    temporary_math = AlmanacMath.new
    twenty_eight_mansions_number \
      = temporary_math.fmod((twenty_eight_mansions_jd + 11.0), 28.0).to_i + 1
    if 27 < twenty_eight_mansions_number
      twenty_eight_mansions_number -= 28
    end

    return twenty_eight_mansions_number
  end
end
