# -*- coding: UTF-8 -*-
require "date"
require "curses"
require 'AlmanacDateTime'
require 'HentsuStar'
require 'FortuneTellingMessage'

#= ニセ四柱推命占い メイン・プログラム
#== 結婚運・金運・健康運・性格 に関する占いを行う
#=== 年の干支も、時の干支も考慮していないし
#=== 所謂四柱推命占いの一種と言って良いのかどうかは
#=== 作った本人からして、実に「？？？」
#=== 四柱推命占いファンの方々、あまり「四柱推命占い」か
#=== どうかでは、突っ込まないでやってください。
#
#Authors::   Yoshihiko Hara(GoroNeko)
#Date::      2010/08/25
#Version::   0.9.0
#Copyright:: Copyright (C) Yoshihiko Hara(GoroNeko), 2010. All rights reserved.
#License::   GNU GENERAL PUBLIC LICENSE Version 2
#

# 本スクリプトの、名前とバージョン
NAME = "PseudoFourPillarsOfDestiny"
VERSION = "0.9.0"

def print_usage
  p "[usage]"
  p "> ruby PseudoFourPillarsOfDestiny.rb [year( 0 < yyyy < 10000 )] [month( 0 < mm < 13 )] [day( 0 < dd < 32)]"
end

#
# --- << main >> ---
#

target_year = 99
target_month = 99
target_day = 99

# ヘルプの表示を指定されていた場合は
# usage の表示のみを行い、処理を終了する
first_arg = ARGV[0].to_s
if "-?" == first_arg or
   "-h" == first_arg or
   "--help" == first_arg
  print_usage
  exit
end

# 処理に対し有効なパラメタ
# （ 第一引数 西暦 1 年 ～ 9999 年、
#    第二引数 1 月 ～ 12 月、
#    第三引数 1 日 ～ 31 日 ）が指定されていた場合、
# 指定年月日の四柱推命（結婚運・金運・健康運・性格）占いを行い
# 結果をコンソールに表示する

# 数値化できる第一引数 ～ 第三引数が存在しなければ
# usage の表示のみ実施
begin
  target_year = ARGV[0].to_i
  target_month = ARGV[1].to_i
  target_day = ARGV[2].to_i
rescue
  print_usage
  exit
end

# 第一引数 ～ 第三引数に指定された値が、それぞれ妥当な年・月・日
# 情報でない場合は、usage の表示のみ実施
if 9999 < target_year or target_year < 1 or
   12 < target_month or target_month < 1 or
   31 < target_day or target_day < 1
  print_usage
  exit
end
first = Date.new(target_year, target_month, 1)
if target_day > end_of_month = ((first >> 1) - 1).day
  print_usage
  exit
end

# 第一引数 ～ 第三引数を占う対象の年月日情報として、
# 変通星を得る
targe_hentsu_star = HentsuStar.new
targe_hentsu_star_no \
 = targe_hentsu_star.get_hentsu_star_no(target_year, target_month, target_day)

# 得られた変通星に対応する占いメッセージを
# 直接コンソールに表示する
# コンソールへの表示には、Curses ライブラリを使用する

# コンソールの初期化
# ※ Curses ライブラリを使用
Curses::init_screen

# 占いメッセージの取得
Curses::addstr("【結婚運】\n")
Curses::addstr(FortuneTellingMessage::MARRIAGE_FATE_MESSAGE[targe_hentsu_star_no])
Curses::addstr("【金運】\n")
Curses::addstr(FortuneTellingMessage::ECONOMIC_FORTUNE_MESSAGE[targe_hentsu_star_no])
Curses::addstr("【健康運】\n")
Curses::addstr(FortuneTellingMessage::HEALTHY_FATE_MESSAGE[targe_hentsu_star_no])
Curses::addstr("【性格】\n")
Curses::addstr(FortuneTellingMessage::CHARACTER_MESSAGE[targe_hentsu_star_no])

# コンソール画面に表示を実施
Curses::refresh
Curses::getch
Curses::close_screen
